//---------------------------------------------------------------------------
// EditCommands.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Math.hpp>
#include "EditCommands.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace EditCommands {
//---------------------------------------------------------------------------
void __fastcall TEditCommands::Initialize()
{
	try {
		FCommands = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TEditCommands::Uninitialize()
{
	delete FCommands;
	FCommands = NULL;
}
//---------------------------------------------------------------------------
__fastcall TEditCommands::TEditCommands() :
	TObject(),
	FCommands(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TEditCommands::~TEditCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TEditCommands::Add(TEditCommand* ACommand)
{
	FCommands->Add(ACommand);
}
//---------------------------------------------------------------------------
void __fastcall TEditCommands::Clear()
{
	FCommands->Clear();
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TEditCommands::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	TFileTimes newFileTimes = AFileTimes;
	try {
		for(int i = 0; i < FCommands->Count; i++) {
			newFileTimes = static_cast<TEditCommand*>(FCommands->Items[i])->Execute(AFileTimeKinds, newFileTimes);
			FCommands->Items[i] = NULL;
		}
	}
	__finally {
		FCommands->Clear();
	}
	return newFileTimes;
}
//---------------------------------------------------------------------------
void __fastcall TEditCommand::Cancel()
{
	throw EEditCancel("The edit command is canceled.");
}
//---------------------------------------------------------------------------
void __fastcall TEditCommand::EnsureValidFileTime(const FILETIME& AFileTime)
{
	if(!IsValidFileTime(AFileTime)) {
		throw EEditError("The file time is not valid.");
	}
}
//---------------------------------------------------------------------------
__fastcall TEditCommand::TEditCommand() :
	TObject(),
	FExecuted(false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TEditCommand::~TEditCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TEditCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	if(FExecuted) {
		throw EEditError("The command has been already executed.");
	}
	FExecuted = true;

	TFileTimes newFileTimes = AFileTimes;
	for(int i = 0; i < TFileTimes::Count; i++) {
		if(!AFileTimeKinds.Contains(static_cast<TFileTimeKind>(i))) {
			continue;
		}
		newFileTimes.Items[i] = Edit(static_cast<TFileTimeKind>(i), newFileTimes.Items[i]);
	}
	return newFileTimes;
}
//---------------------------------------------------------------------------
FILETIME __fastcall TAddCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	EnsureValidFileTime(AFileTime);
	FILETIME newFileTime = DwordLongToFileTime(FileTimeToDwordLong(AFileTime) + FAmount);
	DWORDLONG oldValue = FileTimeToDwordLong(AFileTime);
	DWORDLONG newValue = FileTimeToDwordLong(newFileTime);
	if(!IsValidFileTime(newFileTime) ||
		(FAmount < 0 && oldValue < newValue) ||
		(FAmount > 0 && oldValue > newValue)) {
		throw EEditError("The program cannot calculate the file time.");
	}
	return newFileTime;
}
//---------------------------------------------------------------------------
bool __fastcall TAddCommand::DoGet(LONGLONG& AAmount)
{
	bool canContinue = true;
	if(FOnGet != NULL) {
		FOnGet(this, AAmount, canContinue);
	}
	return canContinue;
}
//---------------------------------------------------------------------------
__fastcall TAddCommand::TAddCommand(LONGLONG AAmount, TGetEvent AOnGet) :
	TEditCommand(),
	FAmount(AAmount),
	FOnGet(AOnGet)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TAddCommand::~TAddCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TAddCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	if(!DoGet(FAmount)) {
		Cancel();
	}
	return TEditCommand::Execute(AFileTimeKinds, AFileTimes);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TConvertCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	EnsureValidFileTime(AFileTime);
	FILETIME newFileTime = {0};
	if(!TryConvertFileTime(AFileTime, FFromLocalTime, FToLocalTime, newFileTime)) {
		throw EEditError("The program cannot convert the file time.");
	}
	return newFileTime;
}
//---------------------------------------------------------------------------
__fastcall TConvertCommand::TConvertCommand(bool AFromLocalTime, bool AToLocalTime) :
	TEditCommand(),
	FFromLocalTime(AFromLocalTime),
	FToLocalTime(AToLocalTime)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TConvertCommand::~TConvertCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TConvertCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	return TEditCommand::Execute(AFileTimeKinds, AFileTimes);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TEnterCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	if(!FHasFileTime) {
		FFileTime = AFileTime;
		if(!DoEnter(FFileTime)) {
			Cancel();
		}
		FHasFileTime = true;
	}
	return FFileTime;
}
//---------------------------------------------------------------------------
bool __fastcall TEnterCommand::DoEnter(FILETIME& AFileTime)
{
	bool canContinue = true;
	if(FOnEnter != NULL) {
		FOnEnter(this, AFileTime, canContinue);
	}
	return canContinue;
}
//---------------------------------------------------------------------------
__fastcall TEnterCommand::TEnterCommand(TEnterEvent AOnEnter) :
	TEditCommand(),
	FFileTime(),
	FHasFileTime(false),
	FOnEnter(AOnEnter)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TEnterCommand::~TEnterCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TEnterCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	return TEditCommand::Execute(AFileTimeKinds, AFileTimes);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TGetCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	return AFileTime;
}
//---------------------------------------------------------------------------
bool __fastcall TGetCommand::DoGet(AnsiString& AFileName)
{
	bool canContinue = true;
	if(FOnGet != NULL) {
		FOnGet(this, AFileName, canContinue);
	}
	return canContinue;
}
//---------------------------------------------------------------------------
__fastcall TGetCommand::TGetCommand(const AnsiString& AFileName, TGetEvent AOnGet) :
	TEditCommand(),
	FFileName(AFileName),
	FOnGet(AOnGet)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TGetCommand::~TGetCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TGetCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	if(!DoGet(FFileName)) {
		Cancel();
	}
	TFileTimes contextFileTimes;
	if(!GetFileTimes(FFileName, AFileTimeKinds, contextFileTimes)) {
		throw EEditError("The program cannot get file times.");
	}
	return TEditCommand::Execute(AFileTimeKinds, TFileTimes(AFileTimes, AFileTimeKinds, contextFileTimes));
}
//---------------------------------------------------------------------------
FILETIME __fastcall TRandomizeTimeCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	EnsureValidFileTime(AFileTime);
	DWORDLONG date =
		FileTimeToDwordLong(AFileTime) -
		FileTimeToDwordLong(AFileTime) % FileTimeIntervalsPerDay;
	DWORDLONG time =
		FileTimeIntervalsPerHour * RandomRange(0, 23) +
		FileTimeIntervalsPerMinute * RandomRange(0, 59) +
		FileTimeIntervalsPerSecond * RandomRange(0, 59) +
		FileTimeIntervalsPerMillisecond * RandomRange(0, 999) +
		FileTimeIntervalsPerMicrosecond * RandomRange(0, 999) +
		RandomRange(0, 9);
	FILETIME newFileTime = DwordLongToFileTime(date + time);
	EnsureValidFileTime(newFileTime);
	return newFileTime;
}
//---------------------------------------------------------------------------
__fastcall TRandomizeTimeCommand::TRandomizeTimeCommand() :
	TEditCommand()
{
	static bool initialized = false;

	if(!initialized) {
		Randomize();
		initialized = true;
	}
}
//---------------------------------------------------------------------------
__fastcall TRandomizeTimeCommand::~TRandomizeTimeCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TRandomizeTimeCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	return TEditCommand::Execute(AFileTimeKinds, AFileTimes);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TResetTimeCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	EnsureValidFileTime(AFileTime);
	FILETIME newFileTime = DwordLongToFileTime(
		FileTimeToDwordLong(AFileTime) -
		FileTimeToDwordLong(AFileTime) % FileTimeIntervalsPerDay);
	EnsureValidFileTime(newFileTime);
	return newFileTime;
}
//---------------------------------------------------------------------------
__fastcall TResetTimeCommand::TResetTimeCommand() :
	TEditCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TResetTimeCommand::~TResetTimeCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TResetTimeCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	return TEditCommand::Execute(AFileTimeKinds, AFileTimes);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TSetCommand::Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime)
{
	return AFileTime;
}
//---------------------------------------------------------------------------
__fastcall TSetCommand::TSetCommand(const TFileTimes& AFileTimes) :
	TEditCommand(),
	FFileTimes(AFileTimes)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSetCommand::~TSetCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TFileTimes __fastcall TSetCommand::Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	return TEditCommand::Execute(AFileTimeKinds, TFileTimes(AFileTimes, AFileTimeKinds, FFileTimes));
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::EditCommands
//---------------------------------------------------------------------------

