//---------------------------------------------------------------------------
#ifndef Moochos_Restamper_EditCommands_H
#define Moochos_Restamper_EditCommands_H
//---------------------------------------------------------------------------
// EditCommands.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include "Lib/Moochos/Moa/Files.h"
#include "Lib/Moochos/Moa/Time.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace EditCommands {
//---------------------------------------------------------------------------
class EEditError : public Exception
{
public:
	__fastcall EEditError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EEditCancel : public EAbort
{
public:
	__fastcall EEditCancel(const AnsiString& AMessage) : EAbort(AMessage) {}
};
//---------------------------------------------------------------------------
class TEditCommand;
class TEditCommands : public TObject
{
private:
	TObjectList* FCommands;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TEditCommands();
	virtual __fastcall ~TEditCommands();
	void __fastcall Add(TEditCommand* ACommand);
	void __fastcall Clear();
	TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TEditCommand : public TObject
{
private:
	bool FExecuted;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime) = 0;
	virtual void __fastcall Cancel();
	virtual void __fastcall EnsureValidFileTime(const FILETIME& AFileTime);
public:
	__fastcall TEditCommand();
	virtual __fastcall ~TEditCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TAddCommand : public TEditCommand
{
public:
	typedef void __fastcall (__closure * TGetEvent)(TObject* ASender, LONGLONG& AAmount, bool& ACanContinue);
private:
	LONGLONG FAmount;
	TGetEvent FOnGet;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
	virtual bool __fastcall DoGet(LONGLONG& AAmount);
public:
	__fastcall TAddCommand(LONGLONG AAmount, TGetEvent AOnGet = NULL);
	virtual __fastcall ~TAddCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TConvertCommand : public TEditCommand
{
private:
	bool FFromLocalTime;
	bool FToLocalTime;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
public:
	__fastcall TConvertCommand(bool AFromLocalTime, bool AToLocalTime);
	virtual __fastcall ~TConvertCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TEnterCommand : public TEditCommand
{
public:
	typedef void __fastcall (__closure * TEnterEvent)(TObject* ASender, FILETIME& AFileTime, bool& ACanContinue);
private:
	FILETIME FFileTime;
	bool FHasFileTime;
	TEnterEvent FOnEnter;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
	virtual bool __fastcall DoEnter(FILETIME& AFileTime);
public:
	__fastcall TEnterCommand(TEnterEvent AOnEnter);
	virtual __fastcall ~TEnterCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TGetCommand : public TEditCommand
{
public:
	typedef void __fastcall (__closure * TGetEvent)(TObject* ASender, AnsiString& AFileName, bool& ACanContinue);
private:
	AnsiString FFileName;
	TGetEvent FOnGet;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
	virtual bool __fastcall DoGet(AnsiString& AFileName);
public:
	__fastcall TGetCommand(const AnsiString& AFileName, TGetEvent AOnGet = NULL);
	virtual __fastcall ~TGetCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TRandomizeTimeCommand : public TEditCommand
{
private:
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
public:
	__fastcall TRandomizeTimeCommand();
	virtual __fastcall ~TRandomizeTimeCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TResetTimeCommand : public TEditCommand
{
private:
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
public:
	__fastcall TResetTimeCommand();
	virtual __fastcall ~TResetTimeCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
class TSetCommand : public TEditCommand
{
private:
	TFileTimes FFileTimes;
protected:
	virtual FILETIME __fastcall Edit(TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
public:
	__fastcall TSetCommand(const TFileTimes& AFileTimes);
	virtual __fastcall ~TSetCommand();
	virtual TFileTimes __fastcall Execute(const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::EditCommands
using namespace Moochos::Restamper::EditCommands;
//---------------------------------------------------------------------------
#endif

