//---------------------------------------------------------------------------
// EditMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Math.hpp>
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Dialogs.h"
#include "EditCommands.h"
#include "EditMan.h"
#include "EnterDialogUnit.h"
#include "FileTimeDialogUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace EditMan {
//---------------------------------------------------------------------------
void __fastcall TEditMan::Initialize()
{
	try {
		FGetDialog = new TOpenDialog(NULL);
		GetDialog->Title = "Get";
		GetDialog->Filter = "All|*";
		GetDialog->Options =
			GetDialog->Options <<
			ofPathMustExist <<
			ofFileMustExist <<
			ofNoDereferenceLinks;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::Uninitialize()
{
	delete FGetDialog;
	FGetDialog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::EnterCommandEnter(TObject* ASender, FILETIME& AFileTime, bool& ACanContinue)
{
	TPtr<TFileTimeDialog> fileTimeDialog = new TFileTimeDialog(NULL);
	fileTimeDialog->FileTime = AFileTime;
	if(!fileTimeDialog->Execute()) {
		ACanContinue = false;
		return;
	}
	AFileTime = fileTimeDialog->FileTime;
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::GetCoomandGet1(TObject* ASender, AnsiString& AFileName, bool& ACanContinue)
{
	GetDialog->FileName = AFileName;
	if(!GetDialog->Execute()) {
		ACanContinue = false;
		return;
	}
	AFileName = GetDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::GetCoomandGet2(TObject* ASender, AnsiString& AFileName, bool& ACanContinue)
{
	AnsiString directory = AFileName;
	if(!ChooseDirectory("Choose the source.", directory)) {
		ACanContinue = false;
		return;
	}
	AFileName = directory;
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::AddCommandGet(TObject* ASender, LONGLONG& AAmount, bool& ACanContinue)
{
	int days = 0;
	{
		AnsiString daysString = days;
		if(!EnterValue("Add", "Specify days to add.", daysString)) {
			ACanContinue = false;
			return;
		}
		if(!TryStrToInt(daysString, days)) {
			throw EEditManError("The value must be an integer.");
		}
	}
	AAmount = FileTimeIntervalsPerDay * days;
}
//---------------------------------------------------------------------------
__fastcall TEditMan::TEditMan(TValueHistory<TFileTimes>* AFileTimesHistory) :
	TObject(),
	FFileTimesHistory(AFileTimesHistory),
	FGetDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TEditMan::~TEditMan()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::EnterCreated(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TEnterCommand(&EnterCommandEnter));
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds * (TFileTimeKinds() << ftCreationTime), FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::EnterLastModified(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TEnterCommand(&EnterCommandEnter));
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds * (TFileTimeKinds() << ftLastWriteTime), FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::EnterLastAccessed(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TEnterCommand(&EnterCommandEnter));
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds * (TFileTimeKinds() << ftLastAccessTime), FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::EnterAllTimes(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TEnterCommand(&EnterCommandEnter));
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds * (TFileTimeKinds() << ftCreationTime << ftLastWriteTime << ftLastAccessTime), FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::GetFromFile(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TGetCommand(AnsiString(), &GetCoomandGet1));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::GetFromDir(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TGetCommand(GetCurrentDir(), &GetCoomandGet2));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::ResetToCreated(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TSetCommand(TFileTimes(FileTimesHistory->Current.Items[ftCreationTime])));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::ResetToLastModified(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TSetCommand(TFileTimes(FileTimesHistory->Current.Items[ftLastWriteTime])));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::ResetToLastAccessed(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TSetCommand(TFileTimes(FileTimesHistory->Current.Items[ftLastAccessTime])));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::SetToNow(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TSetCommand(TFileTimes(SystemTimeToFileTime(GetSystemTime()))));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::IncrementDate(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TAddCommand(FileTimeIntervalsPerDay));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::DecrementDate(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TAddCommand(FileTimeIntervalsPerDay * -1));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::AddDays(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TAddCommand(0, &AddCommandGet));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::ResetTime(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TResetTimeCommand());
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::RandomizeTime(const TFileTimeKinds& AFileTimeKinds, bool ALocalTime)
{
	TPtr<TEditCommands> commands = new TEditCommands();
	commands->Add(new TConvertCommand(false, ALocalTime));
	commands->Add(new TRandomizeTimeCommand());
	commands->Add(new TConvertCommand(ALocalTime, false));
	FileTimesHistory->Current = commands->Execute(AFileTimeKinds, FileTimesHistory->Current);
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::Undo()
{
	FileTimesHistory->Undo();
}
//---------------------------------------------------------------------------
bool __fastcall TEditMan::CanUndo()
{
	return FileTimesHistory->CanUndo();
}
//---------------------------------------------------------------------------
void __fastcall TEditMan::Redo()
{
	FileTimesHistory->Redo();
}
//---------------------------------------------------------------------------
bool __fastcall TEditMan::CanRedo()
{
	return FileTimesHistory->CanRedo();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::EditMan
//---------------------------------------------------------------------------

