//---------------------------------------------------------------------------
// FileTimeDialogUnit.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Lib/Moochos/Moa/Time.h"
#include "FileTimeDialogUnit.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TFileTimeDialog* FileTimeDialog = NULL;
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	if(ModalResult != mrOk) {
		return;
	}

	try {
		int year = Edit11->Text.ToInt();
		if(year < 1601 || year > 30828) {
			throw EFileTimeDialogError("The year must be between 1601 and 30828.");
		}

		int month = Edit12->Text.ToInt();
		if(month < 1 || month > 12) {
			throw EFileTimeDialogError("The month must be between 1 and 12.");
		}

		int day = Edit13->Text.ToInt();
		if(day < 1 || day > 31) {
			throw EFileTimeDialogError("The day must be between 1 and 31.");
		}

		int hour = Edit21->Text.ToInt();
		if(hour < 0 || hour > 23) {
			throw EFileTimeDialogError("The hour must be between 0 and 23.");
		}

		int minute = Edit22->Text.ToInt();
		if(minute < 0 || minute > 59) {
			throw EFileTimeDialogError("The minute must be between 0 and 59.");
		}

		int second = Edit23->Text.ToInt();
		if(second < 0 || second > 59) {
			throw EFileTimeDialogError("The second must be between 0 and 59.");
		}

		AnsiString rest = Edit24->Text;
		if(rest.Length() < 7) {
			rest += AnsiString::StringOfChar('0', 7 - rest.Length());
		}
		else if(rest.Length() == 7) {
			// Do nothing
		}
		else {
			throw EFileTimeDialogError("The decimal part of the second must be at most 7 digits.");
		}

		int milliseconds = rest.SubString(1, 3).ToInt();
		if(milliseconds < 0 || milliseconds > 999) {
			throw EFileTimeDialogError("The milliseconds must be between 0 and 999.");
		}

		int microseconds = rest.SubString(4, 3).ToInt();
		if(microseconds < 0 || microseconds > 999) {
			throw EFileTimeDialogError("The microseconds must be between 0 and 999.");
		}

		int nanoseconds = rest.SubString(7, 1).ToInt() * 100;
		if(nanoseconds < 0 || nanoseconds > 900) {
			throw EFileTimeDialogError("The nanoseconds must be between 0 and 900 by 100.");
		}

		FILETIME newFileTime = MakeFileTime(
			year,
			month,
			day,
			hour,
			minute,
			second,
			milliseconds,
			microseconds,
			nanoseconds);
		if(!IsValidFileTime(newFileTime)) {
			throw EFileTimeDialogError("Specify a valid date and time.");
		}
		FileTime = newFileTime;
	}
	catch(EConvertError& e) {
		throw EFileTimeDialogError("Specify a valid date and time.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::Prepare()
{
	FileTime = FileTime;

	FBackup.FileTime = FileTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::Unprepare()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::Restore()
{
	FileTime = FBackup.FileTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::Process()
{
	ActiveControl = NULL;
	SelectFirst();
	if(ShowModal() != mrOk) {
		Abort();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileTimeDialog::SetFileTime(const FILETIME& AFileTime)
{
	FFileTime = AFileTime;
	WORD parts[9] = {0};
	ProcessFileTime(
		AFileTime,
		parts[0],
		parts[1],
		parts[2],
		parts[3],
		parts[4],
		parts[5],
		parts[6],
		parts[7],
		parts[8]);
	Edit11->Text = parts[0];
	Edit12->Text = parts[1];
	Edit13->Text = parts[2];
	Edit21->Text = parts[3];
	Edit22->Text = parts[4];
	Edit23->Text = parts[5];
	Edit24->Text = AnsiString().sprintf("%07u",
		static_cast<unsigned int>(parts[6]) * 1000 * 10 +
		static_cast<unsigned int>(parts[7]) * 10 +
		static_cast<unsigned int>(parts[8]) / 100);
}
//---------------------------------------------------------------------------
__fastcall TFileTimeDialog::TFileTimeDialog(TComponent* AOwner) :
	TForm(AOwner),
	FFileTime(MakeFileTime(0, 0)),
	FBackup()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileTimeDialog::~TFileTimeDialog()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TFileTimeDialog::Execute()
{
	try {
		Prepare();
		Process();
		Unprepare();
	}
	catch(EAbort& e) {
		Restore();
		Unprepare();
		return false;
	}
	catch(...) {
		Unprepare();
		throw;
	}
	return true;
}
//---------------------------------------------------------------------------

