//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Files_H
#define Moochos_Moa_Files_H
//---------------------------------------------------------------------------
// Files.h
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Files {
//---------------------------------------------------------------------------
class EFileError : public Exception
{
public:
	__fastcall EFileError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TFileNameList : public TStringList
{
private:
	__property bool CaseSensitive = {};
protected:
	virtual int __fastcall CompareStrings(const AnsiString S1, const AnsiString S2);
public:
	__fastcall TFileNameList();
	virtual __fastcall ~TFileNameList();
};
//---------------------------------------------------------------------------
class TFileInfoEnumerator
	// Enumerates file informations.
{
private:
	HANDLE FHandle;
	WIN32_FIND_DATA FData;
	bool FFound;
	__fastcall TFileInfoEnumerator(const TFileInfoEnumerator& AOther); // Forbidden
	TFileInfoEnumerator& __fastcall operator =(const TFileInfoEnumerator& AOther); // Forbidden
	void __fastcall FindFirst(const AnsiString& AFileName);
	void __fastcall FindNext();
	void __fastcall FindClose();
protected:
public:
	__fastcall TFileInfoEnumerator(const AnsiString& AFileName);
	__fastcall ~TFileInfoEnumerator();
	bool __fastcall HasNext();
	WIN32_FIND_DATA __fastcall GetNext();
	void __fastcall GetNext(WIN32_FIND_DATA& ANext);
};
//---------------------------------------------------------------------------
extern AnsiString __fastcall ExtractLongPathName(const AnsiString AFileName);
extern bool __fastcall DriveExists(const AnsiString& AFileName);
extern bool __fastcall FileOrDirExists(const AnsiString& AFileName);
extern bool __fastcall IsFileNameMask(const AnsiString& AFileName);
extern bool __fastcall IsAbsoluteFileName(const AnsiString& AFileName);
extern bool __fastcall IsRelativeFileName(const AnsiString& AFileName);
extern bool __fastcall IsDirectory(const WIN32_FIND_DATA& AFindData);
extern bool __fastcall IsDotFileName(const WIN32_FIND_DATA& AFindData);
extern bool __fastcall IsDotDotFileName(const WIN32_FIND_DATA& AFindData);
//---------------------------------------------------------------------------
enum TFileTimeKind
{
	ftCreationTime,
	ftLastWriteTime,
	ftLastAccessTime,
};
typedef Set<TFileTimeKind, ftCreationTime, ftLastAccessTime> TFileTimeKinds;
//---------------------------------------------------------------------------
struct TFileTimes
{
	static const int Count = 3;
	FILETIME Items[Count]; // The index corresponds to TFileTimeKind
	__fastcall TFileTimes();
	__fastcall TFileTimes(const TFileTimes& ABase, const TFileTimeKinds& AContextKinds, const TFileTimes& AContext);
	__fastcall TFileTimes(const FILETIME& AFileTime);
	__fastcall TFileTimes(const FILETIME& ACreationTime, const FILETIME& ALastWriteTime, const FILETIME& ALastAccessTime);
};
//---------------------------------------------------------------------------
extern bool __fastcall GetFileTimes(const AnsiString& AFileName, const TFileTimeKinds& AFileTimeKinds, TFileTimes& AFileTimes);
extern bool __fastcall SetFileTimes(const AnsiString& AFileName, const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes);

extern FILETIME __fastcall GetItem(const TFileTimes& AFileTimes, TFileTimeKind AFileTimeKind);
extern void __fastcall SetItem(TFileTimes& AFileTimes, TFileTimeKind AFileTimeKind, const FILETIME& AFileTime);
//---------------------------------------------------------------------------
// Functors
//---------------------------------------------------------------------------
struct TIsFileNameLess
{
	inline bool operator ()(const AnsiString& AFileName1, const AnsiString& AFileName2) const;
};
//---------------------------------------------------------------------------
// Implementations
//---------------------------------------------------------------------------
inline bool TIsFileNameLess::operator ()(const AnsiString& AFileName1, const AnsiString& AFileName2) const
{
	return AnsiCompareFileName(AFileName1, AFileName2) < 0;
}
//---------------------------------------------------------------------------
}}} // End of Moochos::Moa::Files
using namespace Moochos::Moa::Files;
//---------------------------------------------------------------------------
#endif

