//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Histories_H
#define Moochos_Moa_Histories_H
//---------------------------------------------------------------------------
// Histories.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vector>
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Histories {
//---------------------------------------------------------------------------
class EHistoryError : public Exception
{
public:
	__fastcall EHistoryError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
template <typename T>
class TValueHistory : public TObject
{
private:
	std::vector<T> FValues;
	int FPosition;
	TNotifyEvent FOnChange;
	void __fastcall DoChange();
	T __fastcall GetCurrent();
	void __fastcall SetCurrent(const T& AValue);
	T __fastcall GetPrevious();
	T __fastcall GetNext();
	T __fastcall GetValue(int AIndex);
	int __fastcall GetCount();
protected:
public:
	__fastcall TValueHistory(const T& AValue);
	virtual __fastcall ~TValueHistory();
	void __fastcall Replace(const T& AValue);
	void __fastcall Clear();
	void __fastcall Reset(const T& AValue);
	void __fastcall Assign(TValueHistory<T>* AOther);
	void __fastcall Undo();
	bool __fastcall CanUndo();
	void __fastcall Redo();
	bool __fastcall CanRedo();
	__property T Current = {read=GetCurrent, write=SetCurrent};
	__property T Previous = {read=GetPrevious};
	__property T Next = {read=GetNext};
	__property T Values[int AIndex] = {read=GetValue};
	__property int Count = {read=GetCount};
	__property int Position = {read=FPosition};
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Implementations
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
template <typename T>
T __fastcall TValueHistory<T>::GetCurrent()
{
	return FValues[Position];
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::SetCurrent(const T& AValue)
{
	FValues.resize(Position + 2);
	FPosition++;
	FValues[Position] = AValue;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
T __fastcall TValueHistory<T>::GetPrevious()
{
	if(!CanUndo()) {
		throw EHistoryError("The previous value is not available.");
	}
	return FValues[Position - 1];
}
//---------------------------------------------------------------------------
template <typename T>
T __fastcall TValueHistory<T>::GetNext()
{
	if(!CanRedo()) {
		throw EHistoryError("The next value is not available.");
	}
	return FValues[Position + 1];
}
//---------------------------------------------------------------------------
template <typename T>
T __fastcall TValueHistory<T>::GetValue(int AIndex)
{
	return FValues[AIndex];
}
//---------------------------------------------------------------------------
template <typename T>
int __fastcall TValueHistory<T>::GetCount()
{
	return FValues.size();
}
//---------------------------------------------------------------------------
template <typename T>
__fastcall TValueHistory<T>::TValueHistory(const T& AValue) :
	TObject(),
	FValues(),
	FPosition(-1),
	FOnChange(NULL)
{
	FValues.push_back(AValue);
	FPosition = 0;
}
//---------------------------------------------------------------------------
template <typename T>
__fastcall TValueHistory<T>::~TValueHistory()
{
	// Do nothing
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Replace(const T& AValue)
{
	FValues[Position] = AValue;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Clear()
{
	Reset(FValues[FPosition]);
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Reset(const T& AValue)
{
	FValues[0] = AValue;
	FPosition = 0;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Assign(TValueHistory<T>* AOther)
{
	if(AOther == this) {
		return;
	}
	FValues = AOther->FValues;
	FPosition = AOther->FPosition;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Undo()
{
	if(!CanUndo()) {
		throw EHistoryError("The undo operation is not available.");
	}
	FPosition--;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
bool __fastcall TValueHistory<T>::CanUndo()
{
	return Position > 0;
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TValueHistory<T>::Redo()
{
	if(!CanRedo()) {
		throw EHistoryError("The redo operation is not available.");
	}
	FPosition++;
	DoChange();
}
//---------------------------------------------------------------------------
template <typename T>
bool __fastcall TValueHistory<T>::CanRedo()
{
	return Position < Count - 1;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Histories
using namespace Moochos::Moa::Histories;
//---------------------------------------------------------------------------
#endif

