//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Lists_H
#define Moochos_Moa_Lists_H
//---------------------------------------------------------------------------
// Lists.h
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Lists {
//---------------------------------------------------------------------------
// TList
//---------------------------------------------------------------------------
extern void __fastcall AddItems(TList* ASrcList, TList* ADstList);
extern void __fastcall Invert(TList* AList);
//---------------------------------------------------------------------------
// TObjectList
//---------------------------------------------------------------------------
class TAutoObjectListOwnsObjects
	// Temporarily changes OwnsObject of a TObjectList object until this
	// object is freed.
{
private:
	TObjectList* FList;
	bool FOwnsObjects;
	__fastcall TAutoObjectListOwnsObjects(const TAutoObjectListOwnsObjects& AOther); // Forbedden
	TAutoObjectListOwnsObjects& __fastcall operator =(const TAutoObjectListOwnsObjects& AOther); // Forbedden
protected:
public:
	__fastcall TAutoObjectListOwnsObjects(TObjectList* AList, bool AOwnsObjects);
	__fastcall ~TAutoObjectListOwnsObjects();
};
//---------------------------------------------------------------------------
// TStrings
//---------------------------------------------------------------------------
class TAutoStringsUpdate
	// Calls BeginUpdate on construction and EndUpdate on destruction.
{
private:
	TStrings* FList;
	__fastcall TAutoStringsUpdate(const TAutoStringsUpdate& AOther); // Forbedden
	TAutoStringsUpdate& __fastcall operator =(const TAutoStringsUpdate& AOther); // Forbedden
protected:
public:
	__fastcall TAutoStringsUpdate(TStrings* AList);
	__fastcall ~TAutoStringsUpdate();
};
//---------------------------------------------------------------------------
extern bool __fastcall TryAdd(TStrings* AList, const AnsiString& AString);
extern bool __fastcall TryAddObject(TStrings* AList, const AnsiString& AString, TObject* AObject);
extern void __fastcall Resize(TStrings* AList, int ACount);
extern void __fastcall Pack(TStrings* AList, bool AByString = true, bool AByObject = false);
extern void __fastcall Invert(TStrings* AList);
extern AnsiString __fastcall PopFirst(TStrings* AList);
extern AnsiString __fastcall PopLast(TStrings* AList);
extern AnsiString __fastcall First(TStrings* AList);
extern AnsiString __fastcall Last(TStrings* AList);
//---------------------------------------------------------------------------
// TStringList
//---------------------------------------------------------------------------
extern int __fastcall Find(TStringList* AStringList, const AnsiString& AString);
//---------------------------------------------------------------------------
// DynamicArray
//---------------------------------------------------------------------------
template <typename T>
extern void __fastcall Invert(DynamicArray<T>& AArray);
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Implementations
//---------------------------------------------------------------------------
template <typename T>
void __fastcall Invert(DynamicArray<T>& AArray)
{
	T temp;
	for(int i = 0; i < AArray.Length / 2; i++) {
		temp = AArray[i];
		AArray[i] = AArray[AArray.Length - 1 - i];
		AArray[AArray.Length - 1 - i] = temp;
	}
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Lists
using namespace Moochos::Moa::Lists;
//---------------------------------------------------------------------------
#endif

