//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Sets_H
#define Moochos_Moa_Sets_H
//---------------------------------------------------------------------------
// Sets.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <values.h>
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Sets {
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall And(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall Or(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall Not(const Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall Xor(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern unsigned char __fastcall GetMinElement(const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern unsigned char __fastcall GetMaxElement(const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern int __fastcall GetLength(const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern AnsiString __fastcall SetToStr(const Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall StrToSet(const AnsiString& AString, const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall StrToSetDef(const AnsiString& AString, const Set<T, MinEl, MaxEl>& ADefault);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern bool __fastcall TryStrToSet(const AnsiString& AString, Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern int __fastcall SetToInt(const Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern int __fastcall SetToIntDef(const Set<T, MinEl, MaxEl>& ASet, int ADefault);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern bool __fastcall TrySetToInt(const Set<T, MinEl, MaxEl>& ASet, int& AInt);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall IntToSet(int AInt, const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall IntToSetDef(int AInt, const Set<T, MinEl, MaxEl>& ADefault);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern bool __fastcall TryIntToSet(int AInt, Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern __int64 __fastcall SetToInt64(const Set<T, MinEl, MaxEl>& ASet);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern __int64 __fastcall SetToInt64Def(const Set<T, MinEl, MaxEl>& ASet, __int64 ADefault);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern bool __fastcall TrySetToInt64(const Set<T, MinEl, MaxEl>& ASet, __int64& AInt64);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall Int64ToSet(__int64 AInt64, const Set<T, MinEl, MaxEl>* AType = NULL);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern Set<T, MinEl, MaxEl> __fastcall Int64ToSetDef(__int64 AInt64, const Set<T, MinEl, MaxEl>& ADefault);

template <typename T, unsigned char MinEl, unsigned char MaxEl>
extern bool __fastcall TryInt64ToSet(__int64 AInt64, Set<T, MinEl, MaxEl>& ASet);
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Implementations
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall And(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2)
{
	return ASet1 * ASet2;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall Or(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2)
{
	return ASet1 + ASet2;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall Not(const Set<T, MinEl, MaxEl>& ASet)
{
	Set<T, MinEl, MaxEl> newSet;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if(!ASet.Contains(static_cast<T>(i))) {
			newSet << static_cast<T>(MinEl + i);
		}
	}
	return newSet;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall Xor(const Set<T, MinEl, MaxEl>& ASet1, const Set<T, MinEl, MaxEl>& ASet2)
{
	Set<T, MinEl, MaxEl> newSet;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if((ASet1.Contains(static_cast<T>(i)) ? 1 : 0) == (ASet2.Contains(static_cast<T>(i)) ? 1 : 0)) {
			newSet << static_cast<T>(MinEl + i);
		}
	}
	return newSet;
	//
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
unsigned char __fastcall GetMinElement(const Set<T, MinEl, MaxEl>* AType)
{
	return MinEl;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
unsigned char __fastcall GetMaxElement(const Set<T, MinEl, MaxEl>* AType)
{
	return MaxEl;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
int __fastcall GetLength(const Set<T, MinEl, MaxEl>* AType)
{
	return MaxEl - MinEl + 1;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
AnsiString __fastcall SetToStr(const Set<T, MinEl, MaxEl>& ASet)
{
	AnsiString stringValue = AnsiString::StringOfChar('0', MaxEl - MinEl + 1);
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if(ASet.Contains(static_cast<T>(i))) {
			stringValue[i + 1] = '1';
		}
	}
	return stringValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall StrToSet(const AnsiString& AString, const Set<T, MinEl, MaxEl>* AType)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryStrToSet(AString, setValue)) {
		throw EConvertError("The program cannot convert a string to a set.");
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall StrToSetDef(const AnsiString& AString, const Set<T, MinEl, MaxEl>& ADefault)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryStrToSet(AString, setValue)) {
		return ADefault;
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
bool __fastcall TryStrToSet(const AnsiString& AString, Set<T, MinEl, MaxEl>& ASet)
{
	if(AString.Length() < MaxEl - MinEl + 1) {
		return false;
	}
	Set<T, MinEl, MaxEl> setValue;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if(AString[i + 1] == '1') {
			setValue << static_cast<T>(MinEl + i);
		}
	}
	ASet = setValue;
	return true;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
int __fastcall SetToInt(const Set<T, MinEl, MaxEl>& ASet)
{
	int intValue;
	if(!TrySetToInt(ASet, intValue)) {
		throw EConvertError("The program cannot convert a set to an integer.");
	}
	return intValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
int __fastcall SetToIntDef(const Set<T, MinEl, MaxEl>& ASet, int ADefault)
{
	int intValue;
	if(!TrySetToInt(ASet, intValue)) {
		return ADefault;
	}
	return intValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
bool __fastcall TrySetToInt(const Set<T, MinEl, MaxEl>& ASet, int& AInt)
{
	struct Local
	{
		// Avoid warnings on compile time
		static bool CanAccept(int ACapacity, int AMin, int AMax)
		{
			return ACapacity >= AMax - AMin + 1;
		}
	};

	if(!Local::CanAccept(BITSPERBYTE * sizeof(int), MinEl, MaxEl)) {
		return false;
	}
	int intValue;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if(ASet.Contains(static_cast<T>(MinEl + i))) {
			intValue |= (1 << i);
		}
	}
	AInt = intValue;
	return true;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall IntToSet(int AInt, const Set<T, MinEl, MaxEl>* AType)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryIntToSet(AInt, setValue)) {
		throw EConvertError("The program cannot convert an integer to a set.");
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall IntToSetDef(int AInt, const Set<T, MinEl, MaxEl>& ADefault)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryIntToSet(AInt, setValue)) {
		return ADefault;
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
bool __fastcall TryIntToSet(int AInt, Set<T, MinEl, MaxEl>& ASet)
{
	struct Local
	{
		// Avoid warnings on compile time
		static bool CanAccept(int ACapacity, int AMin, int AMax)
		{
			return ACapacity >= AMax - AMin + 1;
		}
	};

	if(!Local::CanAccept(BITSPERBYTE * sizeof(int), MinEl, MaxEl)) {
		return false;
	}
	Set<T, MinEl, MaxEl> setValue;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if((AInt & (1 << i)) != 0) {
			setValue << static_cast<T>(MinEl + i);
		}
	}
	ASet = setValue;
	return true;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
__int64 __fastcall SetToInt64(const Set<T, MinEl, MaxEl>& ASet)
{
	__int64 int64Value;
	if(!TrySetToInt64(ASet, int64Value)) {
		throw EConvertError("The program cannot convert a set to an integer.");
	}
	return int64Value;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
__int64 __fastcall SetToInt64Def(const Set<T, MinEl, MaxEl>& ASet, __int64 ADefault)
{
	__int64 int64Value;
	if(!TrySetToInt64(ASet, int64Value)) {
		return ADefault;
	}
	return int64Value;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
bool __fastcall TrySetToInt64(const Set<T, MinEl, MaxEl>& ASet, __int64& AInt64)
{
	if(BITSPERBYTE * sizeof(__int64) < MaxEl - MinEl + 1) {
		return false;
	}
	__int64 int64Value;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if(ASet.Contains(static_cast<T>(MinEl + i))) {
			int64Value |= (1 << i);
		}
	}
	AInt64 = int64Value;
	return true;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall Int64ToSet(__int64 AInt64, const Set<T, MinEl, MaxEl>* AType)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryInt64ToSet(AInt64, setValue)) {
		throw EConvertError("The program cannot convert an integer to a set.");
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
Set<T, MinEl, MaxEl> __fastcall Int64ToSetDef(__int64 AInt64, const Set<T, MinEl, MaxEl>& ADefault)
{
	Set<T, MinEl, MaxEl> setValue;
	if(!TryInt64ToSet(AInt64, setValue)) {
		return ADefault;
	}
	return setValue;
}
//---------------------------------------------------------------------------
template <typename T, unsigned char MinEl, unsigned char MaxEl>
bool __fastcall TryInt64ToSet(__int64 AInt64, Set<T, MinEl, MaxEl>& ASet)
{
	if(BITSPERBYTE * sizeof(__int64) < MaxEl - MinEl + 1) {
		return false;
	}
	Set<T, MinEl, MaxEl> setValue;
	for(unsigned char i = MinEl; i <= MaxEl; i++) {
		if((AInt64 & (1 << i)) != 0) {
			setValue << static_cast<T>(MinEl + i);
		}
	}
	ASet = setValue;
	return true;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Sets
using namespace Moochos::Moa::Sets;
//---------------------------------------------------------------------------
#endif

