//---------------------------------------------------------------------------
// Tasks.h
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Tasks {
//---------------------------------------------------------------------------
void __fastcall TTask::TTaskThread::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::TTaskThread::Execute()
{
	DoExecute();
}
//---------------------------------------------------------------------------
__fastcall TTask::TTaskThread::TTaskThread(bool ACreateSuspended) :
	TThread(ACreateSuspended),
	FOnExecute(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::TTaskThread::~TTaskThread()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::TTaskThread::ExecuteSynchronize(TThreadMethod AMethod)
{
	Synchronize(AMethod);
}
//---------------------------------------------------------------------------
void __fastcall TTask::ExecuteMain()
{
	try {
		try {
			Main();
			End();
		}
		catch(Exception& e) {
			DoException(&e);
		}
		catch(...) {
			try {
				throw Exception("An unknown exception occured.");
			}
			catch(Exception& e) {
				DoException(&e);
			}
		}
	}
	catch(...) {
		if(Async) {
			// Do not throw any exception from a sub-thread
		}
		else {
			throw;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Start()
{
	Synchronize(StartInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TTask::StartInMainThread(void* AData)
{
	FStarted = true;
	DoStart();
}
//---------------------------------------------------------------------------
void __fastcall TTask::End()
{
	Synchronize(EndInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TTask::EndInMainThread(void* AData)
{
	FEnded = true;
	DoEnd();
}
//---------------------------------------------------------------------------
void __fastcall TTask::CancelInMainThread(void* AData)
{
	if(!IsBusy() || Canceled) {
		return;
	}
	FCanceled = true;
	DoCancel();
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadExecute(TObject* ASender)
{
	ExecuteMain();
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizeInMainThread()
{
	FSyncMethod(FSyncData);
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoStartInMainThread(void* AData)
{
	if(OnStart != NULL) {
		OnStart(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoEndInMainThread(void* AData)
{
	if(OnEnd != NULL) {
		OnEnd(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgressInMainThread(void* AData)
{
	if(OnProgress != NULL) {
		OnProgress(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoCancelInMainThread(void* AData)
{
	if(OnCancel != NULL) {
		OnCancel(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoIdleInMainThread(void* AData)
{
	if(OnIdle != NULL) {
		OnIdle(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoExceptionInMainThread(void* AData)
{
	if(OnException != NULL) {
		OnException(this, static_cast<Exception*>(FSyncData));
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::SetProgressInMainThread(void* AData)
{
	int newProgress = *static_cast<int*>(AData);
	if(newProgress == Progress) {
		return;
	}
	FProgress = newProgress;
	DoProgress();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Idle()
{
	DoIdle();
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoStart()
{
	if(OnStart != NULL) {
		Synchronize(DoStartInMainThread, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoEnd()
{
	if(OnEnd != NULL) {
		Synchronize(DoEndInMainThread, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgress()
{
	if(OnProgress != NULL) {
		Synchronize(DoProgressInMainThread, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoCancel()
{
	if(OnCancel != NULL) {
		Synchronize(DoCancelInMainThread, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoIdle()
{
	if(OnIdle == NULL) {
		return;
	}
	Synchronize(DoIdleInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoException(Exception* AException)
{
	if(OnException != NULL) {
		Synchronize(DoExceptionInMainThread, AException);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::SetProgress(int AProgress)
{
	if(AProgress == FProgress) {
		return;
	}
	Synchronize(SetProgressInMainThread, &AProgress);
}
//---------------------------------------------------------------------------
void __fastcall TTask::Synchronize(TSyncMethod AMethod, void* AData)
{
	if(GetCurrentThreadId() == MainThreadID) {
		AMethod(AData);
	}
	else {
		FSyncMethod = AMethod;
		FSyncData = AData;
		FThread->ExecuteSynchronize(SynchronizeInMainThread);
	}
}
//---------------------------------------------------------------------------
__fastcall TTask::TTask(bool AAsync) :
	TObject(),
	FThread(NULL),
	FProgress(-1),
	FAsync(AAsync),
	FStarted(false),
	FEnded(false),
	FCanceled(false),
	FSyncMethod(NULL),
	FSyncData(NULL),
	FOnStart(NULL),
	FOnEnd(NULL),
	FOnProgress(NULL),
	FOnCancel(NULL),
	FOnIdle(NULL),
	FOnException(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::~TTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::Execute()
{
	if(Started) {
		throw ETaskError("The task is tried to execute for multiple times.");
	}
	Start();
	if(Async) {
		FThread = new TTaskThread(true);
		FThread->FreeOnTerminate = true;
		FThread->OnExecute = ThreadExecute;
		FThread->Resume();
	}
	else {
		ExecuteMain();
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Cancel()
{
	Synchronize(CancelInMainThread, NULL);
}
//---------------------------------------------------------------------------
bool __fastcall TTask::IsBusy()
{
	return Started && !Ended;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Tasks
//---------------------------------------------------------------------------

