//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Tasks_H
#define Moochos_Moa_Tasks_H
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
// Tasks.h
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Tasks {
//---------------------------------------------------------------------------
class ETaskError : public Exception
{
public:
	__fastcall ETaskError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TTask : public TObject
	// Represents a task.
{
public:
	typedef void __fastcall (__closure * TExceptionEvent)(TObject* ASender, Exception* AException);
private:
	typedef void __fastcall (__closure * TSyncMethod)(void* AData);
	class TTaskThread : public TThread
	{
	private:
		TNotifyEvent FOnExecute;
		void __fastcall DoExecute();
	protected:
		virtual void __fastcall Execute();
	public:
		__fastcall TTaskThread(bool ACreateSuspended);
		virtual __fastcall ~TTaskThread();
		void __fastcall ExecuteSynchronize(TThreadMethod AMethod);
		__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
	};
	TTaskThread* FThread;
	int FProgress;
	bool FAsync;
	bool FStarted;
	bool FEnded;
	bool FCanceled;
	TSyncMethod FSyncMethod;
	void* FSyncData;
	TNotifyEvent FOnStart;
	TNotifyEvent FOnEnd;
	TNotifyEvent FOnProgress;
	TNotifyEvent FOnCancel;
	TNotifyEvent FOnIdle;
	TExceptionEvent FOnException;
	void __fastcall ExecuteMain();
	void __fastcall Start();
	void __fastcall StartInMainThread(void* AData);
	void __fastcall End();
	void __fastcall EndInMainThread(void* AData);
	void __fastcall CancelInMainThread(void* AData);
	void __fastcall ThreadExecute(TObject* ASender);
	void __fastcall SynchronizeInMainThread();
	void __fastcall DoStartInMainThread(void* AData);
	void __fastcall DoEndInMainThread(void* AData);
	void __fastcall DoProgressInMainThread(void* AData);
	void __fastcall DoCancelInMainThread(void* AData);
	void __fastcall DoIdleInMainThread(void* AData);
	void __fastcall DoExceptionInMainThread(void* AData);
	void __fastcall SetProgressInMainThread(void* AData);
protected:
	virtual void __fastcall Main() = 0;
	virtual void __fastcall Idle();
	virtual void __fastcall DoStart();
	virtual void __fastcall DoEnd();
	virtual void __fastcall DoProgress();
	virtual void __fastcall DoCancel();
	virtual void __fastcall DoIdle();
	virtual void __fastcall DoException(Exception* AException);
	virtual void __fastcall SetProgress(int AProgress);
	virtual void __fastcall Synchronize(TSyncMethod AMethod, void* AData);
public:
	__fastcall TTask(bool AAsync);
	virtual __fastcall ~TTask();
	void __fastcall Execute();
	void __fastcall Cancel();
	bool __fastcall IsBusy();
	__property int Progress = {read=FProgress};
	__property bool Async = {read=FAsync};
	__property bool Started = {read=FStarted};
	__property bool Ended = {read=FEnded};
	__property bool Canceled = {read=FCanceled};
	__property TNotifyEvent OnStart = {read=FOnStart, write=FOnStart};
	__property TNotifyEvent OnEnd = {read=FOnEnd, write=FOnEnd};
	__property TNotifyEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TNotifyEvent OnCancel = {read=FOnCancel, write=FOnCancel};
	__property TNotifyEvent OnIdle = {read=FOnIdle, write=FOnIdle};
	__property TExceptionEvent OnException = {read=FOnException, write=FOnException};
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Tasks
using namespace Moochos::Moa::Tasks;
//---------------------------------------------------------------------------
#endif

