//---------------------------------------------------------------------------
// Time.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Time.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Time {
//---------------------------------------------------------------------------
SYSTEMTIME __fastcall GetSystemTime()
{
	SYSTEMTIME systemTime = {0};
	::GetSystemTime(&systemTime);
	return systemTime;
}
//---------------------------------------------------------------------------
SYSTEMTIME __fastcall MakeSystemTime(WORD AYear, WORD AMonth, WORD ADay, WORD AHour, WORD AMinute, WORD ASecond, WORD AMilliseconds)
{
	SYSTEMTIME systemTime = {0};
	systemTime.wYear = AYear;
	systemTime.wMonth = AMonth;
	systemTime.wDay = ADay;
	systemTime.wHour = AHour;
	systemTime.wMinute = AMinute;
	systemTime.wSecond = ASecond;
	systemTime.wMilliseconds = AMilliseconds;
	return systemTime;
}
//---------------------------------------------------------------------------
bool __fastcall IsValidSystemTime(const SYSTEMTIME& ASystemTime)
{
	if(
		ASystemTime.wYear < 1601 ||
		ASystemTime.wMonth < 1 || ASystemTime.wMonth > 12 ||
		ASystemTime.wDay < 1 || ASystemTime.wDay > 31 ||
		ASystemTime.wHour > 23 ||
		ASystemTime.wMinute > 59 ||
		ASystemTime.wSecond > 59 ||
		ASystemTime.wMilliseconds > 999) {
		return false;
	}
	FILETIME fileTime = {0};
	if(!::SystemTimeToFileTime(&ASystemTime, &fileTime)) {
		return false;
	}
	return FileTimeToDwordLong(fileTime) < 0x8000000000000000;
}
//---------------------------------------------------------------------------
const DWORDLONG FileTimeIntervalsPerMicrosecond = 10;
const DWORDLONG FileTimeIntervalsPerMillisecond = FileTimeIntervalsPerMicrosecond * 1000;
const DWORDLONG FileTimeIntervalsPerSecond = FileTimeIntervalsPerMillisecond * 1000;
const DWORDLONG FileTimeIntervalsPerMinute = FileTimeIntervalsPerSecond * 60;
const DWORDLONG FileTimeIntervalsPerHour = FileTimeIntervalsPerMinute * 60;
const DWORDLONG FileTimeIntervalsPerDay = FileTimeIntervalsPerHour * 24;
//---------------------------------------------------------------------------
FILETIME __fastcall SystemTimeToFileTime(const SYSTEMTIME& ASystemTime)
{
	FILETIME fileTime = {0};
	if(!TrySystemTimeToFileTime(ASystemTime, fileTime)) {
		throw EConvertError("The program convert the system time to a file time.");
	}
	return fileTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall SystemTimeToFileTimeDef(const SYSTEMTIME& ASystemTime, const FILETIME& ADefault)
{
	FILETIME fileTime = {0};
	if(!TrySystemTimeToFileTime(ASystemTime, fileTime)) {
		return ADefault;
	}
	return fileTime;
}
//---------------------------------------------------------------------------
bool __fastcall TrySystemTimeToFileTime(const SYSTEMTIME& ASystemTime, FILETIME& AFileTime)
{
	if(!IsValidSystemTime(ASystemTime)) {
		return false;
	}
	return ::SystemTimeToFileTime(&ASystemTime, &AFileTime);
}
//---------------------------------------------------------------------------
SYSTEMTIME __fastcall FileTimeToSystemTime(const FILETIME& AFileTime)
{
	SYSTEMTIME systemTime = {0};
	if(!TryFileTimeToSystemTime(AFileTime, systemTime)) {
		throw EConvertError("The program cannot convert the system time to a file time.");
	}
	return systemTime;
}
//---------------------------------------------------------------------------
SYSTEMTIME __fastcall FileTimeToSystemTimeDef(const FILETIME& AFileTime, const SYSTEMTIME& ADefault)
{
	SYSTEMTIME systemTime = {0};
	if(!TryFileTimeToSystemTime(AFileTime, systemTime)) {
		return ADefault;
	}
	return systemTime;
}
//---------------------------------------------------------------------------
bool __fastcall TryFileTimeToSystemTime(const FILETIME& AFileTime, SYSTEMTIME& ASystemTime)
{
	if(!IsValidFileTime(AFileTime)) {
		return false;
	}
	return ::FileTimeToSystemTime(&AFileTime, &ASystemTime);
}
//---------------------------------------------------------------------------
FILETIME __fastcall ConvertFileTime(const FILETIME& AFileTime, bool AFromLocal, bool AToLocal)
{
	FILETIME newFileTime = {0};
	if(!TryConvertFileTime(AFileTime, AFromLocal, AToLocal, newFileTime)) {
		throw EConvertError("The program cannot convert the file time.");
	}
	return newFileTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall ConvertFileTimeDef(const FILETIME& AFileTime, bool AFromLocal, bool AToLocal, const FILETIME& ADefault)
{
	FILETIME newFileTime = {0};
	if(!TryConvertFileTime(AFileTime, AFromLocal, AToLocal, newFileTime)) {
		return ADefault;
	}
	return newFileTime;
}
//---------------------------------------------------------------------------
bool __fastcall TryConvertFileTime(const FILETIME& AFileTime, bool AFromLocal, bool AToLocal, FILETIME& ANewFileTime)
{
	if(!IsValidFileTime(AFileTime)) {
		return false;
	}
	FILETIME newFileTime = {0};
	if(AFromLocal && !AToLocal) {
		if(!::LocalFileTimeToFileTime(&AFileTime, &newFileTime)) {
			return false;
		}
	}
	else if(!AFromLocal && AToLocal) {
		if(!::FileTimeToLocalFileTime(&AFileTime, &newFileTime)) {
			return false;
		}
	}
	else {
		newFileTime = AFileTime;
	}
	if(!IsValidFileTime(newFileTime)) {
		return false;
	}
	ANewFileTime = newFileTime;
	return true;
}
//---------------------------------------------------------------------------
DWORDLONG __fastcall FileTimeToDwordLong(const FILETIME& AFileTime)
{
	ULARGE_INTEGER value = {0};
	value.LowPart = AFileTime.dwLowDateTime;
	value.HighPart = AFileTime.dwHighDateTime;
	return value.QuadPart;
}
//---------------------------------------------------------------------------
FILETIME __fastcall DwordLongToFileTime(DWORDLONG ADwordLong)
{
	ULARGE_INTEGER value = {0};
	value.QuadPart = ADwordLong;

	FILETIME fileTime = {0};
	fileTime.dwLowDateTime = value.LowPart;
	fileTime.dwHighDateTime = value.HighPart;
	return fileTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall MakeFileTime(DWORD ALow, DWORD AHigh)
{
	FILETIME fileTime = {0};
	fileTime.dwLowDateTime = ALow;
	fileTime.dwHighDateTime = AHigh;
	return fileTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall MakeFileTime(
	WORD AYear,
	WORD AMonth,
	WORD ADay,
	WORD AHour,
	WORD AMinute,
	WORD ASecond,
	WORD AMilliseconds,
	WORD AMicroseconds,
	WORD ANanoseconds)
{
	if(AMicroseconds > 999 || ANanoseconds > 999) {
		throw ETimeError("The program cannot make a file time.");
	}
	return DwordLongToFileTime(FileTimeToDwordLong(SystemTimeToFileTime(MakeSystemTime(
		AYear,
		AMonth,
		ADay,
		AHour,
		AMinute,
		ASecond,
		AMilliseconds))) +
		10 * static_cast<DWORDLONG>(AMicroseconds) +
		static_cast<DWORDLONG>(ANanoseconds) / 100);
}
//---------------------------------------------------------------------------
void __fastcall ProcessFileTime(
	const FILETIME& AFileTime,
	WORD& AYear,
	WORD& AMonth,
	WORD& ADay,
	WORD& AHour,
	WORD& AMinute,
	WORD& ASecond,
	WORD& AMilliseconds,
	WORD& AMicroseconds,
	WORD& ANanoseconds)
{
	SYSTEMTIME systemTime = FileTimeToSystemTime(AFileTime);
	AYear = systemTime.wYear;
	AMonth = systemTime.wMonth;
	ADay = systemTime.wDay;
	AHour = systemTime.wHour;
	AMinute = systemTime.wMinute;
	ASecond = systemTime.wSecond;
	AMilliseconds = systemTime.wMilliseconds;

	DWORDLONG rest = FileTimeToDwordLong(AFileTime) - FileTimeToDwordLong(SystemTimeToFileTime(systemTime));
	AMicroseconds = rest / 10;
	ANanoseconds = (rest % 10) * 100;
}
//---------------------------------------------------------------------------
bool __fastcall IsValidFileTime(const FILETIME& AFileTime)
{
	return FileTimeToDwordLong(AFileTime) < 0x8000000000000000;
}
//---------------------------------------------------------------------------
bool __fastcall IsUnknownFileTime(const FILETIME& AFileTime)
{
	return AFileTime.dwLowDateTime == 0 && AFileTime.dwHighDateTime == 0;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Time
//---------------------------------------------------------------------------

