//---------------------------------------------------------------------------
// SettingsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/IniFiles.h"
#include "Lib/Moochos/Moa/Sets.h"
#include "MainFormUnit.h"
#include "SettingsMan.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace SettingsMan {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString SettingsFileName = ExtractFilePath(Application->ExeName) + "Settings.ini";

const AnsiString idCreationTime = "CreationTime";
const AnsiString idCurrentDir = "CurrentDir";
const AnsiString idEditMan = "EditMan";
const AnsiString idFilesDialog = "FilesDialog";
const AnsiString idFileTimeKinds = "FileTimeKinds";
const AnsiString idFileTimes = "FileTimes";
const AnsiString idFilterIndex = "FilterIndex";
const AnsiString idGetDialog = "GetDialog";
const AnsiString idLastAccessTime = "LastAccessTime";
const AnsiString idLastWriteTime = "LastWriteTime";
const AnsiString idLeft = "Left";
const AnsiString idListDialog = "ListDialog";
const AnsiString idLocalTime = "LocalTime";
const AnsiString idMainForm = "MainForm";
const AnsiString idOptions = "Options";
const AnsiString idOutputDir = "OutputDir";
const AnsiString idRestoreDialog = "RestoreDialog";
const AnsiString idStampMan = "StampMan";
const AnsiString idSystem = "System";
const AnsiString idTaskIndex = "TaskIndex";
const AnsiString idTop = "Top";

const AnsiString chSeparator = ".";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadMainFormSection(TVarIniFile* AIniFile)
{
	MainForm->Left = AIniFile->ReadInteger(
		idMainForm,
		idLeft,
		MainForm->Left);

	MainForm->Top = AIniFile->ReadInteger(
		idMainForm,
		idTop,
		MainForm->Top);

	MainForm->FileTimeKinds = StrToSet(AIniFile->ReadQuotedString(
		idMainForm,
		idFileTimeKinds,
		SetToStr(MainForm->FileTimeKinds)), static_cast<TFileTimeKinds*>(NULL));

	{
		TFileTimes newFileTimes = MainForm->FileTimes;

		newFileTimes.Items[ftCreationTime] = DwordLongToFileTime(AIniFile->ReadInteger64(
			idMainForm,
			idFileTimes + chSeparator + idCreationTime,
			FileTimeToDwordLong(newFileTimes.Items[ftCreationTime])));

		newFileTimes.Items[ftLastWriteTime] = DwordLongToFileTime(AIniFile->ReadInteger64(
			idMainForm,
			idFileTimes + chSeparator + idLastWriteTime,
			FileTimeToDwordLong(newFileTimes.Items[ftLastWriteTime])));

		newFileTimes.Items[ftLastAccessTime] = DwordLongToFileTime(AIniFile->ReadInteger64(
			idMainForm,
			idFileTimes + chSeparator + idLastAccessTime,
			FileTimeToDwordLong(newFileTimes.Items[ftLastAccessTime])));

		MainForm->FileTimes = newFileTimes;
	}

	MainForm->LocalTime = AIniFile->ReadBool(
		idMainForm,
		idLocalTime,
		MainForm->LocalTime);

	MainForm->Options = StrToSet(AIniFile->ReadQuotedString(
		idMainForm,
		idOptions,
		SetToStr(MainForm->Options)), static_cast<TStampTaskOptions*>(NULL));

	MainForm->StampMan->OutputDir = AIniFile->ReadQuotedString(
		idMainForm,
		idStampMan + chSeparator + idOutputDir,
		MainForm->StampMan->OutputDir);

	MainForm->StampMan->TaskIndex = AIniFile->ReadInteger(
		idMainForm,
		idStampMan + chSeparator + idTaskIndex,
		MainForm->StampMan->TaskIndex);

	MainForm->StampMan->FilesDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idStampMan + chSeparator + idFilesDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->FilesDialog->FilterIndex);

	MainForm->StampMan->ListDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idStampMan + chSeparator + idListDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->ListDialog->FilterIndex);

	MainForm->StampMan->RestoreDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idStampMan + chSeparator + idRestoreDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->RestoreDialog->FilterIndex);

	MainForm->EditMan->GetDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idEditMan + chSeparator + idGetDialog + chSeparator + idFilterIndex,
		MainForm->EditMan->GetDialog->FilterIndex);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveMainFormSection(TVarIniFile* AIniFile)
{
	AIniFile->WriteInteger(
		idMainForm,
		idLeft,
		MainForm->Left);

	AIniFile->WriteInteger(
		idMainForm,
		idTop,
		MainForm->Top);

	AIniFile->WriteQuotedString(
		idMainForm,
		idFileTimeKinds,
		SetToStr(MainForm->FileTimeKinds));

	{
		TFileTimes curFileTimes = MainForm->FileTimes;

		AIniFile->WriteInteger64(
			idMainForm,
			idFileTimes + chSeparator + idCreationTime,
			FileTimeToDwordLong(curFileTimes.Items[ftCreationTime]));

		AIniFile->WriteInteger64(
			idMainForm,
			idFileTimes + chSeparator + idLastWriteTime,
			FileTimeToDwordLong(curFileTimes.Items[ftLastWriteTime]));

		AIniFile->WriteInteger64(
			idMainForm,
			idFileTimes + chSeparator + idLastAccessTime,
			FileTimeToDwordLong(curFileTimes.Items[ftLastAccessTime]));
	}

	AIniFile->WriteBool(
		idMainForm,
		idLocalTime,
		MainForm->LocalTime);

	AIniFile->WriteQuotedString(
		idMainForm,
		idOptions,
		SetToStr(MainForm->Options));

	AIniFile->WriteQuotedString(
		idMainForm,
		idStampMan + chSeparator + idOutputDir,
		MainForm->StampMan->OutputDir);

	AIniFile->WriteInteger(
		idMainForm,
		idStampMan + chSeparator + idTaskIndex,
		MainForm->StampMan->TaskIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idStampMan + chSeparator + idFilesDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->FilesDialog->FilterIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idStampMan + chSeparator + idListDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->ListDialog->FilterIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idStampMan + chSeparator + idRestoreDialog + chSeparator + idFilterIndex,
		MainForm->StampMan->RestoreDialog->FilterIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idEditMan + chSeparator + idGetDialog + chSeparator + idFilterIndex,
		MainForm->EditMan->GetDialog->FilterIndex);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadSystemSection(TVarIniFile* AIniFile)
{
	SetCurrentDir(AIniFile->ReadQuotedString(
		idSystem,
		idCurrentDir,
		GetCurrentDir()));
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveSystemSection(TVarIniFile* AIniFile)
{
	AIniFile->WriteQuotedString(
		idSystem,
		idCurrentDir,
		GetCurrentDir());
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::TSettingsMan() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::~TSettingsMan()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Load()
{
	if(!FileExists(SettingsFileName)) {
		return;
	}
	try {
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		LoadMainFormSection(iniFile);
		LoadSystemSection(iniFile);
	}
	catch(EIniFileException& e) {
		throw ESettingsManError("The program cannot load the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Save()
{
	try {
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		SaveMainFormSection(iniFile);
		SaveSystemSection(iniFile);
		iniFile->UpdateFile();
	}
	catch(EIniFileException& e) {
		throw ESettingsManError("The program cannot save the settings.");
	}
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::SettingsMan
//---------------------------------------------------------------------------

