//---------------------------------------------------------------------------
// StampCommand.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Lists.h"
#include "StampCommand.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampCommand {
//---------------------------------------------------------------------------
void __fastcall TStampSource::EnsureNotFinished()
{
	if(IsFinished()) {
		throw EStampError("No stamp command arguments are available.");
	}
}
//---------------------------------------------------------------------------
__fastcall TStampSource::TStampSource(const TStampTaskArgs& AArgs) :
	TObject(),
	FArgs(AArgs)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TStampSource::~TStampSource()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TStampNewSource::Initialize()
{
	try {
		{
			TPtr<TStrings> fileNames = new TStringList();
			fileNames->Text = Args.FileNames;
			for(int i = 0; i < fileNames->Count; i++) {
				FRest.push_back(fileNames->Strings[i]);
			}
		}
		FillCache();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampNewSource::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TStampNewSource::FillCache()
{
	if(FCache.size() > 0) {
		return;
	}
	while(FCache.size() == 0) {
		AnsiString fileNameMask;
		if(FRest.size() > 0) {
			fileNameMask = EnsureCanAcceptFileName(FRest.front());
			FRest.pop_front();
		}
		else if(FSubdirectories.size() > 0) {
			fileNameMask = IncludeTrailingPathDelimiter(FSubdirectories.front()) + "*";
			FSubdirectories.pop_front();
		}
		else {
			return;
		}
		AnsiString filePath = ExtractFilePath(fileNameMask);
		TFileInfoEnumerator fileInfoEnumerator(fileNameMask);
		if(!fileInfoEnumerator.HasNext()) {
			throw EStampError(AnsiString().sprintf(
				"The program cannot the file \"%s\".",
				fileNameMask.c_str()));
		}
		while(fileInfoEnumerator.HasNext()) {
			WIN32_FIND_DATA fileInfo = fileInfoEnumerator.GetNext();
			if(IsDotFileName(fileInfo) || IsDotDotFileName(fileInfo)) {
				continue;
			}
			AnsiString fileName = filePath + fileInfo.cFileName;
			if(!FHistory.insert(fileName).second) {
				continue;
			}
			if(IsDirectory(fileInfo)) {
				if(Args.Options.Contains(stIncludeSubdirectories)) {
					FSubdirectories.push_back(fileName);
				}
				if(Args.Options.Contains(stExcludeDirectories)) {
					continue;
				}
			}
			FCache.push_back(fileName);
		}
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TStampNewSource::EnsureCanAcceptFileName(const AnsiString& AFileName)
{
	AnsiString newFileName;
	if(!IsAbsoluteFileName(AFileName) ||
		IsFileNameMask(AFileName) ||
		(newFileName = ExtractLongPathName(AFileName)).Length() == 0) {
		throw EStampError(AnsiString().sprintf(
			"The program cannot accept the file name \"%s\".",
			AFileName.c_str()));
	}
	return newFileName;
}
//---------------------------------------------------------------------------
__fastcall TStampNewSource::TStampNewSource(const TStampTaskArgs& AArgs) :
	TStampSource(AArgs),
	FHistory(TIsFileNameLess()),
	FRest(),
	FSubdirectories(),
	FCache()
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampNewSource::~TStampNewSource()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TStampCommandArgs __fastcall TStampNewSource::Read()
{
	EnsureNotFinished();
	TStampCommandArgs stampCommandArgs;
	stampCommandArgs.FileName = FCache.front();
	FCache.pop_front();
	stampCommandArgs.FileTimeKinds = Args.FileTimeKinds;
	stampCommandArgs.FileTimes = Args.FileTimes;
	stampCommandArgs.RemoveReadOnlyAttr = Args.Options.Contains(stRemoveReadOnlyAttr);
	if(FCache.size() == 0) {
		FillCache();
	}
	return stampCommandArgs;
}
//---------------------------------------------------------------------------
bool __fastcall TStampNewSource::IsFinished()
{
	return FCache.size() == 0;
}
//---------------------------------------------------------------------------
void __fastcall TStampRestoreSource::Initialize()
{
	try {
		FBackupFile = new TStringList();
		FBackupFile->LoadFromFile(Args.SourceFileName);
	}
	catch(EStreamError& e) {
		Uninitialize();
		throw EStampError("The program cannot load bakup records.");
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampRestoreSource::Uninitialize()
{
	delete FBackupFile;
	FBackupFile = NULL;
}
//---------------------------------------------------------------------------
__fastcall TStampRestoreSource::TStampRestoreSource(const TStampTaskArgs& AArgs) :
	TStampSource(AArgs),
	FBackupFile(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampRestoreSource::~TStampRestoreSource()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TStampCommandArgs __fastcall TStampRestoreSource::Read()
{
	EnsureNotFinished();
	TStampCommandArgs stampCommandArgs;
	try {
		stampCommandArgs = StrToStampCommandArgs(PopLast(FBackupFile));
	}
	catch(EConvertError& e) {
		throw EStampError("There is an invalid backup record.");
	}
	return stampCommandArgs;
}
//---------------------------------------------------------------------------
bool __fastcall TStampRestoreSource::IsFinished()
{
	return FBackupFile->Count == 0;
}
//---------------------------------------------------------------------------
TStampSource* __fastcall MakeStampSource(const TStampTaskArgs& AArgs)
{
	if(AArgs.Options.Contains(stRestore)) {
		return new TStampRestoreSource(AArgs);
	}
	else {
		return new TStampNewSource(AArgs);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampBackup::Initialize()
{
	try {
		FStream = new TFileStream(FArgs.BackupFileName, fmCreate | fmShareDenyWrite);
	}
	catch(EStreamError& e) {
		Uninitialize();
		throw EStampError("The program cannot create a backup file.");
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampBackup::Uninitialize()
{
	delete FStream;
	FStream = NULL;
}
//---------------------------------------------------------------------------
__fastcall TStampBackup::TStampBackup(const TStampTaskArgs& AArgs) :
	TObject(),
	FArgs(AArgs),
	FStream(NULL),
	FLineBreak("\r\n")
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampBackup::~TStampBackup()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TStampBackup::Write(const TStampCommandArgs& AArgs)
{
	try {
		AnsiString line = StampCommandArgsToStr(AArgs) + FLineBreak;
		FStream->Write(line.c_str(), line.Length());
	}
	catch(EConvertError& e) {
		throw EStampError("The program cannot make a backup record.");
	}
	catch(EStreamError& e) {
		throw EStampError("The program cannot write a backup record.");
	}
}
//---------------------------------------------------------------------------
TStampCommandArgs __fastcall TStampCommand::MakeBackupRecord()
{
	DoMakeBackupRecord(false, false);
	TStampCommandArgs backupRecord = Args;
	{
		TFileInfoEnumerator fileInfoEnumerator(backupRecord.FileName);
		if(!fileInfoEnumerator.HasNext()) {
			DoMakeBackupRecord(true, false);
			Abort();
		}
		WIN32_FIND_DATA fileInfo = fileInfoEnumerator.GetNext();
		backupRecord.FileTimes.Items[ftCreationTime] = fileInfo.ftCreationTime;
		backupRecord.FileTimes.Items[ftLastWriteTime] = fileInfo.ftLastWriteTime;
		backupRecord.FileTimes.Items[ftLastAccessTime] = fileInfo.ftLastAccessTime;
	}
	DoMakeBackupRecord(true, true);
	return backupRecord;
}
//---------------------------------------------------------------------------
DWORD __fastcall TStampCommand::GetFileAttrs()
{
	if(!Args.RemoveReadOnlyAttr) {
		return 0;
	}
	DoGetFileAttrs(false, false);
	DWORD fileAttrs = GetFileAttributes(Args.FileName.c_str());
	if(fileAttrs == 0xFFFFFFFF) {
		DoGetFileAttrs(true, false);
		Abort();
	}
	DoGetFileAttrs(true, true);
	return fileAttrs;
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::RemoveReadOnlyAttr(DWORD AFileAttrs)
{
	if(!Args.RemoveReadOnlyAttr || (AFileAttrs & FILE_ATTRIBUTE_READONLY) == 0) {
		return;
	}
	DoRemoveReadOnlyAttr(false, false);
	if(!SetFileAttributes(Args.FileName.c_str(), AFileAttrs & ~FILE_ATTRIBUTE_READONLY)) {
		DoRemoveReadOnlyAttr(true, false);
		Abort();
	}
	DoRemoveReadOnlyAttr(true, true);
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::SetFileTimes()
{
	DoSetFileTimes(false, false);
	if(!Moochos::Moa::Files::SetFileTimes(Args.FileName, Args.FileTimeKinds, Args.FileTimes)) {
		DoSetFileTimes(true, false);
		Abort();
	}
	DoSetFileTimes(true, true);
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::SaveBackupRecord(const TStampCommandArgs& ABackupRecord)
{
	DoSaveBackupRecord(false, false);
	Backup->Write(ABackupRecord);
	DoSaveBackupRecord(true, true);
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::RestoreReadOnlyAttr(DWORD AFileAttrs)
{
	if(!Args.RemoveReadOnlyAttr || (AFileAttrs & FILE_ATTRIBUTE_READONLY) == 0) {
		return;
	}
	DoRestoreReadOnlyAttr(false, false);
	if((AFileAttrs & FILE_ATTRIBUTE_READONLY) != 0 &&
		!SetFileAttributes(Args.FileName.c_str(), AFileAttrs)) {
		DoRestoreReadOnlyAttr(true, false);
		return;
	}
	DoRestoreReadOnlyAttr(true, true);
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::Abort()
{
	throw ECommandAbort("The stamp task is aborted.");
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoMakeBackupRecord(bool ADone, bool ASucceeded)
{
	if(OnMakeBackupRecord != NULL) {
		OnMakeBackupRecord(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoGetFileAttrs(bool ADone, bool ASucceeded)
{
	if(OnGetFileAttrs != NULL) {
		OnGetFileAttrs(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoRemoveReadOnlyAttr(bool ADone, bool ASucceeded)
{
	if(OnRemoveReadOnlyAttr != NULL) {
		OnRemoveReadOnlyAttr(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoSetFileTimes(bool ADone, bool ASucceeded)
{
	if(OnSetFileTimes != NULL) {
		OnSetFileTimes(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoRestoreReadOnlyAttr(bool ADone, bool ASucceeded)
{
	if(OnRestoreReadOnlyAttr != NULL) {
		OnRestoreReadOnlyAttr(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::DoSaveBackupRecord(bool ADone, bool ASucceeded)
{
	if(OnSaveBackupRecord != NULL) {
		OnSaveBackupRecord(this, ADone, ASucceeded);
	}
}
//---------------------------------------------------------------------------
__fastcall TStampCommand::TStampCommand(const TStampCommandArgs& AArgs, TStampBackup* ABackup) :
	TObject(),
	FArgs(AArgs),
	FBackup(ABackup),
	FOnMakeBackupRecord(NULL),
	FOnGetFileAttrs(NULL),
	FOnRemoveReadOnlyAttr(NULL),
	FOnSetFileTimes(NULL),
	FOnRestoreReadOnlyAttr(NULL),
	FOnSaveBackupRecord(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TStampCommand::~TStampCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TStampCommand::Execute()
{
	try {
		TStampCommandArgs backupRecord = MakeBackupRecord();
		DWORD fileAttrs = GetFileAttrs();
		RemoveReadOnlyAttr(fileAttrs);
		try {
			SetFileTimes();
			SaveBackupRecord(backupRecord);
		}
		catch(ECommandAbort& e) {
			// Do nothing
		}
		RestoreReadOnlyAttr(fileAttrs);
	}
	catch(ECommandAbort& e) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::StampCommand
//---------------------------------------------------------------------------

