//---------------------------------------------------------------------------
#ifndef Moochos_Restamper_StampCommand_H
#define Moochos_Restamper_StampCommand_H
//---------------------------------------------------------------------------
// StampCommand.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <list>
#include <set>
#include <Classes.hpp>
#include "Lib/Moochos/Moa/Files.h"
#include "Stamp.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampCommand {
//---------------------------------------------------------------------------
class TStampSource : public TObject
{
private:
	TStampTaskArgs FArgs;
protected:
	void __fastcall EnsureNotFinished();
	__property TStampTaskArgs Args = {read=FArgs};
public:
	__fastcall TStampSource(const TStampTaskArgs& AArgs);
	virtual __fastcall ~TStampSource();
	virtual TStampCommandArgs __fastcall Read() = 0;
	virtual bool __fastcall IsFinished() = 0;
};
//---------------------------------------------------------------------------
class TStampNewSource : public TStampSource
{
private:
	std::set<AnsiString, TIsFileNameLess> FHistory;
	std::list<AnsiString> FRest;
	std::list<AnsiString> FSubdirectories;
	std::list<AnsiString> FCache;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall FillCache();
	AnsiString __fastcall EnsureCanAcceptFileName(const AnsiString& AFileName);
protected:
public:
	__fastcall TStampNewSource(const TStampTaskArgs& AArgs);
	virtual __fastcall ~TStampNewSource();
	virtual TStampCommandArgs __fastcall Read();
	virtual bool __fastcall IsFinished();
};
//---------------------------------------------------------------------------
class TStampRestoreSource : public TStampSource
{
private:
	TStrings* FBackupFile;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TStampRestoreSource(const TStampTaskArgs& AArgs);
	virtual __fastcall ~TStampRestoreSource();
	virtual TStampCommandArgs __fastcall Read();
	virtual bool __fastcall IsFinished();
};
//---------------------------------------------------------------------------
extern TStampSource* __fastcall MakeStampSource(const TStampTaskArgs& AArgs);
//---------------------------------------------------------------------------
class TStampBackup : public TObject
{
private:
	TStampTaskArgs FArgs;
	TStream* FStream;
	AnsiString FLineBreak;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TStampBackup(const TStampTaskArgs& AArgs);
	virtual __fastcall ~TStampBackup();
	void __fastcall Write(const TStampCommandArgs& AArgs);
};
//---------------------------------------------------------------------------
class TStampCommand : public TObject
{
public:
	typedef void __fastcall (__closure * TStepEvent)(TObject* ASender, bool ADone, bool ASucceeded);
private:
	class ECommandAbort : public EAbort
	{
	public:
		__fastcall ECommandAbort(const AnsiString& AMessage) : EAbort(AMessage) {}
	};
	TStampCommandArgs FArgs;
	TStampBackup* FBackup;
	TStepEvent FOnMakeBackupRecord;
	TStepEvent FOnGetFileAttrs;
	TStepEvent FOnRemoveReadOnlyAttr;
	TStepEvent FOnSetFileTimes;
	TStepEvent FOnRestoreReadOnlyAttr;
	TStepEvent FOnSaveBackupRecord;
	TStampCommandArgs __fastcall MakeBackupRecord();
	DWORD __fastcall GetFileAttrs();
	void __fastcall RemoveReadOnlyAttr(DWORD AFileAttrs);
	void __fastcall SetFileTimes();
	void __fastcall SaveBackupRecord(const TStampCommandArgs& ABackupRecord);
	void __fastcall RestoreReadOnlyAttr(DWORD AFileAttrs);
	void __fastcall Abort();
	void __fastcall DoMakeBackupRecord(bool ADone, bool ASucceeded);
	void __fastcall DoGetFileAttrs(bool ADone, bool ASucceeded);
	void __fastcall DoRemoveReadOnlyAttr(bool ADone, bool ASucceeded);
	void __fastcall DoSetFileTimes(bool ADone, bool ASucceeded);
	void __fastcall DoRestoreReadOnlyAttr(bool ADone, bool ASucceeded);
	void __fastcall DoSaveBackupRecord(bool ADone, bool ASucceeded);
protected:
public:
	__fastcall TStampCommand(const TStampCommandArgs& AArgs, TStampBackup* ABackup);
	virtual __fastcall ~TStampCommand();
	void __fastcall Execute();
	__property TStampCommandArgs Args = {read=FArgs};
	__property TStampBackup* Backup = {read=FBackup};
	__property TStepEvent OnMakeBackupRecord = {read=FOnMakeBackupRecord, write=FOnMakeBackupRecord};
	__property TStepEvent OnGetFileAttrs = {read=FOnGetFileAttrs, write=FOnGetFileAttrs};
	__property TStepEvent OnRemoveReadOnlyAttr = {read=FOnRemoveReadOnlyAttr, write=FOnRemoveReadOnlyAttr};
	__property TStepEvent OnSetFileTimes = {read=FOnSetFileTimes, write=FOnSetFileTimes};
	__property TStepEvent OnRestoreReadOnlyAttr = {read=FOnRestoreReadOnlyAttr, write=FOnRestoreReadOnlyAttr};
	__property TStepEvent OnSaveBackupRecord = {read=FOnSaveBackupRecord, write=FOnSaveBackupRecord};
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::StampCommand
using namespace Moochos::Restamper::StampCommand;
//---------------------------------------------------------------------------
#endif

