//---------------------------------------------------------------------------
#ifndef Moochos_Restamper_StampTask_H
#define Moochos_Restamper_StampTask_H
//---------------------------------------------------------------------------
// StampTask.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <list>
#include <Classes.hpp>
#include "Lib/Moochos/Moa/Tasks.h"
#include "Stamp.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampCommand { class TStampBackup; }}} using namespace Moochos::Restamper::StampCommand;
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampTask {
//---------------------------------------------------------------------------
class TStampLog : public TObject
{
private:
	TFileStream* FStream;
	AnsiString FFileName;
	AnsiString FInfoTag;
	AnsiString FWarningTag;
	AnsiString FErrorTag;
	AnsiString FDateTimeFormat;
	AnsiString FLineBreak;
	int FInfoCount;
	int FWarningCount;
	int FErrorCount;
	TNotifyEvent FOnChange;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall DoChange();
	AnsiString __fastcall FormatMessage(const TDateTime& ADateTime, const AnsiString& ATag, const AnsiString& AMessage);
	void __fastcall WriteLine(const AnsiString& AText);
protected:
public:
	__fastcall TStampLog(const AnsiString& AFileName);
	virtual __fastcall ~TStampLog();
	void __fastcall LogInfo(const AnsiString& AMessage);
	void __fastcall LogWarning(const AnsiString& AMessage);
	void __fastcall LogError(const AnsiString& AMessage);
	__property AnsiString FileName = {read=FFileName, write=FFileName};
	__property AnsiString InfoTag = {read=FInfoTag, write=FInfoTag};
	__property AnsiString WarningTag = {read=FWarningTag, write=FWarningTag};
	__property AnsiString ErrorTag = {read=FErrorTag, write=FErrorTag};
	__property AnsiString DateTimeFormat = {read=FDateTimeFormat, write=FDateTimeFormat};
	__property AnsiString LineBreak = {read=FLineBreak, write=FLineBreak};
	__property int InfoCount = {read=FInfoCount};
	__property int WarningCount = {read=FWarningCount};
	__property int ErrorCount = {read=FErrorCount};
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
class TStampTask : public TTask
{
public:
	typedef void __fastcall (__closure * TStatusEvent)(TObject* ASender, const AnsiString& AStatus);
private:
	class ETaskCancel : public EAbort
	{
	public:
		__fastcall ETaskCancel(const AnsiString& AMessage) : EAbort(AMessage) {}
	};
	TStampTaskArgs FArgs;
	TStampLog* FLog;
	TStatusEvent FOnStatus;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall GetCommandArgsList(std::list<TStampCommandArgs>& ACommandArgsList);
	void __fastcall ProcessCommandArgsList(std::list<TStampCommandArgs>& ACommandArgsList);
	void __fastcall ProcessCommandArgs(const TStampCommandArgs& ACommandArgs, TStampBackup* ABackup);
	void __fastcall EnsureCanContinue();
	void __fastcall CommandMakeBackupRecord(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall CommandGetFileAttrs(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall CommandRemoveReadOnlyAttr(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall CommandSetFileTimes(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall CommandRestoreReadOnlyAttr(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall CommandSaveBackupRecord(TObject* ASender, bool ADone, bool ASucceeded);
	void __fastcall DoStatus(const AnsiString& AStatus);
	void __fastcall DoStatusInMainThread(void* AData);
protected:
	virtual void __fastcall Main();
public:
	__fastcall TStampTask(bool AAsync, const TStampTaskArgs& AArgs);
	virtual __fastcall ~TStampTask();
	__property TStampTaskArgs Args = {read=FArgs};
	__property TStampLog* Log = {read=FLog};
	__property TStatusEvent OnStatus = {read=FOnStatus, write=FOnStatus};
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::StampTask
using namespace Moochos::Restamper::StampTask;
//---------------------------------------------------------------------------
#endif

